import logging

from django.views.generic import TemplateView


logger = logging.getLogger(__name__)


class SamlIDPErrorView(TemplateView):
    """ Default error view when a 'known' error
        occurs in the saml2 authentication views.
    """
    template_name = 'error.html'
    status = 403

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        exception = kwargs.get("exception")

        context.update({
            "exception_type": exception.__class__.__name__ if exception else None,
            "exception_msg": str(exception) if exception else None,
            "extra_message": kwargs.get("extra_message"),
        })
        return context

    def render_to_response(self, context, **response_kwargs):
        response_kwargs['status'] = self.status
        return super(TemplateView, self).render_to_response(context, **response_kwargs)



class MetadataNotFound(Exception):
    pass


class MetadataCorruption(Exception):
    pass


class NotYetImplemented(Exception):
    pass


class SPConfigurationMissing(Exception):
    pass


class UnavailableRequiredAttributes(Exception):
    pass


class DisabledSP(Exception):
    pass



from django import forms
from django.contrib.auth.forms import AuthenticationForm
from django.utils.translation import gettext_lazy as _


class LoginForm(AuthenticationForm):
    # these are inherited
    # username = forms.CharField()
    # password = forms.CharField(widget=forms.PasswordInput())
    forget_agreement = forms.BooleanField(label=_("Delete previous agreement"),
                                          required=False,
                                          localize=True)
    forget_login = forms.BooleanField(label=_("Forget access"),
                                      required=False,
                                      localize=True)


class AgreementForm(forms.Form):
    CHOICES = ((1, _('I give my consent')),
               (0, _('I deny my consent')))

    confirm = forms.ChoiceField(choices=CHOICES, widget=forms.RadioSelect)
    dont_show_again = forms.BooleanField(label=_("Don't show this screen "
                                                 "on next login."),
                                         required=False,
                                         localize=True)



import logging

from django.conf import settings
from . base import BaseProcessor
from . unical_attributes_generator import UnicalAttributeGenerator


logger = logging.getLogger(__name__)


if 'ldap_peoples' in settings.INSTALLED_APPS:
    from ldap_peoples.models import LdapAcademiaUser

if 'multildap' in settings.INSTALLED_APPS:
    pass


def _get_username(user):
    if isinstance(user, str):
        username = user
    else:
        username = user.username
    return username


class GroupProcessor(BaseProcessor):
    """
        Example implementation of access control for users:
        - superusers are allowed
        - staff is allowed
        - they have to belong to a certain group
    """
    group = "ExampleGroup"

    def has_access(self, user):  # pragma: no cover
        return user.is_superuser or \
            user.is_staff or \
            user.groups.filter(name=self.group).exists()


class LdapAcademiaProcessor(BaseProcessor):
    """ Processor class used to retrieve attribute from LDAP server
        and user nameID (userID) with standard formats
    """

    def get_identity(self, user):
        username = _get_username(user)
        return LdapAcademiaUser.objects.filter(uid=username).first()

    def create_identity(self, user, sp={}):
        """ Generate an identity dictionary of the user based on the
            given mapping of desired user attributes by the SP
        """
        default_mapping = {'username': 'username'}
        sp_mapping = sp['config'].get('attribute_mapping',
                                      default_mapping)

        # get ldap user
        lu = self.get_identity(user)
        # logging.info("{} doesn't have a valid computed ePPN in LDAP, please fix it!".format(user.username))
        results = self.process_attributes(user, sp_mapping)

        if not lu:
            return results

        results = self.process_attributes(lu, sp_mapping)

        # add custom/legacy attribute made by processing
        results = self.extra_attr_processing(results, sp_mapping)

        # if targetedID is available give it to sp
        if self.eduPersonTargetedID:
            results['eduPersonTargetedID'] = [self.eduPersonTargetedID]

        return results


class LdapUnicalAcademiaProcessor(LdapAcademiaProcessor):
    """
    The same of its father but with a custom attribute processing
    for legacy support to stange SP
    """

    def extra_attr_processing(self, results, sp_mapping):
        return UnicalAttributeGenerator.process(results, sp_mapping)


class LdapUnicalMultiAcademiaProcessor(LdapUnicalAcademiaProcessor):
    """
    Uses pyMultiLDAP to gather an uid from multiple sources.
    It will stop on the first occurrence.
    """

    def get_identity(self, user):
        if hasattr(self, 'saml_request') and hasattr(self.saml_request, 'session'):
            if self.request.saml_session.get('identity_attributes'):
                return type('', (object,), self.request.saml_session['identity_attributes'])()

        # otherwise do another query ...
        username = _get_username(user)
        identity = None
        for lc in settings.LDAP_CONNECTIONS:  # pragma: no coverage
            ldapfilter = '(uid={})'.format(username)
            logging.debug("Processor {} searches for {} in {}".format(self.__class__,
                                                                      username,
                                                                      lc))
            identity = lc.get(search=ldapfilter, format='object')
            if identity:
                return identity



import logging

from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend
# from django.contrib.auth.decorators import user_passes_test
from django.db import connections

from ldap3.utils import conv
from ldap_peoples.models import LdapAcademiaUser

logger = logging.getLogger(__name__)


class LdapAcademiaAuthBackend(ModelBackend):
    """
        This class logout a user if another session of that user
        will be created

        in settings.py
        AUTHENTICATION_BACKENDS = [ 'idp.ldap_auth.backends.ModelBackend',
                                    'ldap_peoples.auth.LdapAcademiaAuthBackend'
                                  ]
    """

    def authenticate(self, request, username=None, password=None):
        ldap_conn = connections['ldap']
        user = None
        username = conv.escape_filter_chars(username, encoding=None)
        lu = LdapAcademiaUser.objects.filter(uid=username).first()
        if not lu:
            logger.info("--- LDAP BIND failed for {} ---".format(username))
            return None

        # check if username exists and if it is active
        try:
            ldap_conn.connect()
            ldap_conn.connection.bind_s(lu.distinguished_name(),
                                        password)
            ldap_conn.connection.unbind_s()
        except Exception:
            logger.info(
                "--- LDAP {} seems to be unable to Auth ---".format(username))
            return None

        # if account beign unlocked this will be always false
        if not lu.is_active():
            logger.info(
                "--- LDAP {} seems to be disabled ---".format(username))
            return None

        # username would be like an EPPN
        username = lu.uid
        try:
            user = get_user_model().objects.get(username=username)
            # update attrs:
            user.email = lu.mail[0]
            user.first_name = lu.cn
            user.last_name = lu.sn
            user.origin = 'ldap_peoples'
            user.save()
        except Exception:
            user = get_user_model().objects.create(username=username,
                                                   email=lu.mail[0],
                                                   first_name=lu.cn,
                                                   last_name=lu.sn,
                                                   origin='ldap_peoples')

        # TODO: Create a middleware for this
        # disconnect already created session, only a session per user is allowed
        # get all the active sessions
        # if not settings.MULTIPLE_USER_AUTH_SESSIONS:
            # for session in Session.objects.all():
            # try:
            # if int(session.get_decoded()['_auth_user_id']) == user.pk:
            # session.delete()
            # except (KeyError, TypeError, ValueError):
            # pass

        return user



import json

from django.core.management.base import BaseCommand
from uniauth_saml2_idp.utils import get_idp_config

from uniauth_saml2_idp.models import MetadataStore


class Command(BaseCommand):
    help = 'Metadata Query protocol'

    def add_arguments(self, parser):
        parser.epilog = 'Example: ./manage.py mdquery -e https://auth.unical.it/idp/metadata/ -f json'
        parser.add_argument('-e', required=True,
                            help="Entity to request metadata for")
        parser.add_argument(
            '-f', default=['json', 'saml2'], help='output format')
        parser.add_argument('-debug', required=False, action="store_true",
                            help="see debug message")

    def handle(self, *args, **options):
        idp = get_idp_config()
        for md in MetadataStore.objects.filter(is_active=1,
                                               is_valid=1):
            if md.type in ('file', 'local'):
                idp.metadata.load(md.type, md.url)
            else:
                idp.metadata.load(md.type,
                                  url=md.url, ca_cert=md.file,
                                  **json.loads(md.kwargs))
        res = idp.metadata[options['e']]
        if options['f'] == 'json':
            print(json.dumps(res, indent=2))
        else:
            print(idp.metadata.dumps())



# Generated by Django 3.0.7 on 2020-08-01 16:51

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='MetadataStore',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=256)),
                ('url', models.CharField(
                    blank=True, help_text='for "remote" and "mdq", use path if "local".', max_length=255, null=True)),
                ('file', models.FileField(
                    blank=True, help_text='https cert if type==mdq https cert if type==remote xml file if type==file', null=True, upload_to='metadata')),
                ('type', models.CharField(choices=[
                 ('remote', 'remote'), ('mdq', 'mdq'), ('local', 'local')], max_length=12)),
                ('kwargs', models.TextField(default='{}', help_text='A dictionary')),
                ('is_valid', models.BooleanField(default=False,
                                                 help_text='if sign validation was succesfull')),
                ('is_active', models.BooleanField(default=False,
                                                  help_text='enable/disable this metadata source')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True,
                                                 help_text='when last download/validation occourred', null=True)),
            ],
            options={
                'verbose_name': 'Metadata Store',
                'verbose_name_plural': 'Metadatas Store',
            },
        ),
        migrations.CreateModel(
            name='ServiceProvider',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('entity_id', models.CharField(max_length=254, unique=True)),
                ('display_name', models.CharField(max_length=254)),
                ('metadata_url', models.URLField(blank=True, default='',
                                                 help_text='optional, usually this is the same of entityID', max_length=254)),
                ('description', models.TextField(blank=True, default='')),
                ('agreement_screen', models.BooleanField(default=True)),
                ('agreement_consent_form', models.BooleanField(default=False)),
                ('agreement_message', models.TextField(blank=True, default='')),
                ('signing_algorithm', models.CharField(choices=[('http://www.w3.org/2000/09/xmldsig#rsa-sha1', 'SIG_RSA_SHA1'), ('http://www.w3.org/2001/04/xmldsig-more#rsa-sha224', 'SIG_RSA_SHA224'), ('http://www.w3.org/2001/04/xmldsig-more#rsa-sha256',
                                                                                                                                                                                                          'SIG_RSA_SHA256'), ('http://www.w3.org/2001/04/xmldsig-more#rsa-sha384', 'SIG_RSA_SHA384'), ('http://www.w3.org/2001/04/xmldsig-more#rsa-sha512', 'SIG_RSA_SHA512')], default='http://www.w3.org/2001/04/xmldsig-more#rsa-sha256', max_length=256)),
                ('digest_algorithm', models.CharField(choices=[('http://www.w3.org/2000/09/xmldsig#sha1', 'DIGEST_SHA1'), ('http://www.w3.org/2001/04/xmldsig-more#sha224', 'DIGEST_SHA224'), ('http://www.w3.org/2001/04/xmlenc#sha256', 'DIGEST_SHA256'), (
                    'http://www.w3.org/2001/04/xmldsig-more#sha384', 'DIGEST_SHA384'), ('http://www.w3.org/2001/04/xmlenc#sha512', 'DIGEST_SHA512'), ('http://www.w3.org/2001/04/xmlenc#ripemd160', 'DIGEST_RIPEMD160')], default='http://www.w3.org/2001/04/xmlenc#sha256', max_length=256)),
                ('disable_encrypted_assertions', models.BooleanField(
                    default=True, help_text='disable encryption')),
                ('attribute_processor', models.CharField(blank=True, default='idp.processors.LdapUnicalMultiAcademiaProcessor',
                                                         help_text='"package.file.classname", example: "idp.processors.LdapAcademiaProcessor"', max_length=256)),
                ('attribute_mapping', models.TextField(
                    blank=True, default='{\n    "cn": "cn",\n    "codice_fiscale": "codice_fiscale",\n    "displayName": "displayName",\n    "eduPersonAffiliation": "eduPersonAffiliation",\n    "eduPersonEntitlement": "eduPersonEntitlement",\n    "eduPersonHomeOrganization": "eduPersonHomeOrganization",\n    "eduPersonPrincipalName": "eduPersonPrincipalName",\n    "eduPersonScopedAffiliation": "eduPersonScopedAffiliation",\n    "eduPersonTargetedID": "eduPersonTargetedID",\n    "email": [\n        "mail",\n        "email"\n    ],\n    "givenName": [\n        "givenName",\n        "another_possible_occourrence"\n    ],\n    "mail": [\n        "mail",\n        "email"\n    ],\n    "matricola_dipendente": "matricola_dipendente",\n    "matricola_studente": "matricola_studente",\n    "schacHomeOrganization": "schacHomeOrganization",\n    "schacPersonalUniqueCode": "schacPersonalUniqueCode",\n    "schacPersonalUniqueID": "schacPersonalUniqueID",\n    "sn": "sn"\n}', help_text='Attribute that would be release to this SP, in JSON format.', null=True)),
                ('force_attribute_release', models.BooleanField(default=False,
                                                                help_text='Release the configured attribute mapping regardless of what SP asks for.')),
                ('is_valid', models.BooleanField(default=False)),
                ('is_active', models.BooleanField(default=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True, null=True)),
                ('last_seen', models.DateTimeField(blank=True, null=True)),
            ],
            options={
                'verbose_name': 'Service Provider',
                'verbose_name_plural': 'Service Providers',
            },
        ),
        migrations.CreateModel(
            name='AgreementRecord',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('sp_entity_id', models.TextField()),
                ('attrs', models.TextField()),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('user', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Agreement Record',
                'verbose_name_plural': 'Agreement Records',
            },
        ),
    ]



import re
import sys

from glob import glob
from setuptools import setup, find_packages

_src_folder = '.'
_pkg_name = 'uniauth_saml2_idp'


def readme():
    with open('README.md') as f:
        return f.read()


def get_requirements(fname='requirements.txt'):
    fopen = open(fname, 'r')
    return fopen.read().splitlines()


setup(
    name="uniauth_saml2_idp",
    version='2.0.0-beta',
    description="uniAuth is a Django SAML2 Identity Provider",
    long_description=readme(),
    long_description_content_type='text/markdown',
    author='Giuseppe De Marco, Francesco Filicetti',
    author_email='giuseppe.demarco@unical.it, francesco.filicetti@unical.it',
    license="Apache 2.0",
    url='https://github.com/UniversitaDellaCalabria/uniAuth',

    packages=[f"{_pkg_name}"],
    package_dir={f"{_pkg_name}": f"{_src_folder}/{_pkg_name}"},

    package_data={f"{_pkg_name}": [i.replace(f'{_src_folder}/{_pkg_name}/', '')
                                   for i in glob(f'{_src_folder}/{_pkg_name}/**',
                                                 recursive=True)]
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 3.0",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: WSGI",
        "Topic :: Security",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        ],
    install_requires=get_requirements(),
    zip_safe=False,
    )



import logging

from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend
from django.conf import settings

from ldap3.utils import conv


logger = logging.getLogger(__name__)


class LdapUnicalMultiAcademiaAuthBackend(ModelBackend):
    """
    This class logout a user if another session of that user
    will be created

    in settings.py
    AUTHENTICATION_BACKENDS = ['idp.multildap_auth.LdapUnicalMultiAcademiaAuthBackend',
                              ]
    """

    def authenticate(self, request, username=None, password=None):
        logger.info("--- LDAP BIND to MultiLDAP proxy ---")

        user = None
        lu = None
        dn = None

        username = conv.escape_filter_chars(username, encoding=None)
        for lc in settings.LDAP_CONNECTIONS:
            search_filter = '(uid={})'.format(username)
            lu = lc.get(search=search_filter)
            if lu:
                logger.info(
                    "--- LDAP search filter {} ---".format(search_filter))
                break

        if not lu:
            logger.info("--- LDAP BIND failed for {} ---".format(username))
            return None

        dn = list(lu.keys())[0]
        lu_obj = lc._as_object(lu)

        # check if username exists and if it is active
        if not lc.authenticate(dn, password):
            msg = "--- LDAP user {} seems to be unable to Auth to {} ---"
            logger.info(msg.format(dn, lc))
            return None
        else:
            msg = "--- LDAP user {} succesfully authenticated to {} ---"
            logger.info(msg.format(dn, lc))
            attrs = ','.join([i for i in list(lu.values())[0]])
            logger.info("--- LDAP user attributes: [{}]".format(attrs))
        username = lu_obj.uid[0]
        user = get_user_model().objects.filter(username=username).first()
        if user:
            # update attrs:
            user.email = lu_obj.mail[0]
            user.first_name = lu_obj.givenName[0]
            user.last_name = lu_obj.sn[0]
            user.origin = lc.__repr__()
            user.save()
        else:
            logger.info("--- Creating user: {}".format(username))
            user = get_user_model().objects.create(username=username,
                                                   email=lu_obj.mail[0],
                                                   first_name=lu_obj.givenName[0],
                                                   last_name=lu_obj.sn[0],
                                                   origin=lc.__repr__())

        # avoids another LDAP query in Attributes processors
        request.saml_session['identity_attributes'] = lu[dn]
        return user
